function z=fval3(x,y)
%FVAL3 is the function for Test 3 in [1].
%
%   The algorithm is described in
%   [1] D. Breda, S. Maset and R. Vermiglio, "An adaptive algorithm for
%   efficient computation of level curves of surfaces"
%   Numerical Algorithms xx (2009), pp.xx-xx.
%   Last modified december 24th, 2008.

global N
for j=1:length(x)
    for k=1:length(y)
        z(j,k)=root(x(j,k),y(j,k),N);
    end
end

%-------------------------------------------------------------------------------------%
%                         CHARACTERISTIC ROOTS SUBROUTINE                             %
%-------------------------------------------------------------------------------------%
% algorithm described in                                                              %
% D. Breda, S. Maset and R. Vermiglio,                                                %
% "Pseudospectral differencing methods for characteristic roots of                    %
% delay differential equations                                                        %
% SIAM J. Sci. Comput., Vol. 27 (2), 482-495 (2005)                                   %
%-------------------------------------------------------------------------------------%
function lambda=root(p1,p2,N)

[L0,L,m,tau,k]=model(p1,p2);
if k==0
    lambda=eig(L0);
    lambda=max(real(lambda));
    return
else
    AN=spig(L0,L,m,tau,k,N);
    lambda=eig(AN);
end
lambda=desort(lambda);
lambda=real(lambda(1));
%-------------------------------------------------------------------------------------%
function AN=spig(L0,L,m,tau,k,N)

AN=zeros(m*(1+N),m*(1+N));
T=max(tau);
[D,x]=cheb(N,T);
AN(m+1:m*(1+N),1:m*(1+N))=kron(D(2:1+N,1:1+N),eye(m));
for i=1:N+1
    if i==1
        G=L0;
    else
        G=zeros(m);
    end
    for l=1:k
        G=G+L(m*(l-1)+1:m*l,1:m)*elle(i-1,-tau(l),x);
    end
    AN(1:m,m*(i-1)+1:m*i)=G;
end
%-------------------------------------------------------------------------------------%
function [D,x]=cheb(N,T)

if N==0
    D=0;
    x=1;
    return
end
x=cos(pi*(0:N)/N)';
x=T*(x-1)/2;
c=[2;ones(N-1,1);2].*(-1).^(0:N)';
X=repmat(x,1,N+1);
dX=X-X';
D=(c*(1./c)')./(dX+(eye(N+1)));
D=D-diag(sum(D'));
%-------------------------------------------------------------------------------------%
function l=elle(i,y,x)

a=prod(y-x(find(x~=x(i+1))));
b=prod(x(i+1)-x(find(x~=x(i+1))));
l=a/b;
%-------------------------------------------------------------------------------------%
function m=desort(x)

A=[real(x),imag(x)];
A=-sortrows(-A,2);
B=-sortrows(-A,1);
m=B(:,1)+i*B(:,2);
%-------------------------------------------------------------------------------------%
%-------------------------------------------------------------------------------------%


%-------------------------------------------------------------------------------------%
%                                 MODEL by Olgac $ Sipahi                             %
%-------------------------------------------------------------------------------------%
function [L0,L,m,tau,k]=model(p1,p2)

a=4e-3;
L0=[0,1,0,0,0,0,0,0;0,0,1,0,0,0,0,0;0,0,0,1,0,0,0,0;...
        0,0,0,0,1,0,0,0;0,0,0,0,0,1,0,0;0,0,0,0,0,0,1,0;0,0,0,0,0,0,0,1;...
    -.5185023765852724441392270e30*a-.1075351657755261522379553e32*a^2-.2430785852880000000000000e29,...
    -.4174170241827182791095290e26*a-2230368827957600000000000.-.5056648129163682083785512e27*a^2,...
    -130121451835793323950597.1*a-1778618859039301543460757.*a^2-8496797224177928000000.000,...
    -587265712220664640.0000000-7054537044830156858.735851*a-42474335299941959137.19098*a^2,...
    -9717574448970745.362798373*a-1058888831058576.179200000-56988625772609101.73824989*a^2,...
    -259052496095.6623185422203*a-49165433086.736,-55256687.96-213360796.2475742265454962*a,-1294];
G1=[0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;...
        0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;...
    .1075351657755261522379553e32*a^2+.2592511882926362220696134e30*a,...
    .2087085120913591395547645e26*a+.5056648129163682083785510e27*a^2,...
    65060725917896661975298.51*a+1778618859039301543460757.*a^2,...
    42474335299941959137.19098*a^2+3527268522415078429.367926*a,...
    4858787224485372.681399187*a+56988625772609101.73824989*a^2,...
    129526248047.8311592711102*a,106680398.1237871132727480*a,0];
G2=G1;
G3=[0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;...
        0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;...
    -.2688379144388153805948885e31*a^2,-.1264162032290920520946378e27*a^2,...
    -444654714759825385865189.0*a^2,-10618583824985489784.29774*a^2,...
    -14247156443152275.43456247*a^2,0,0,0];
G4=G3;
G5=[0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;...
        0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;0,0,0,0,0,0,0,0;...
    -.5376758288776307611897770e31*a^2,-.2528324064581841041892757e27*a^2,...
    -889309429519650771730377.8*a^2,-21237167649970979568.59549*a^2,...
    -28494312886304550.86912494*a^2,0,0,0];
L=[G1;G2;G3;G4;G5];
tau=[p1;p2;2*p1;2*p2;p1+p2];
m=8;
k=5;
for j=1:k
    if tau(j)==0
        L0=L0+L(m*(j-1)+1:m*j,:);
        L(m*(j-1)+1:m*j,:)=zeros(m);
    end
end
[tau,L]=ord(tau,L,m);
f=min(find(tau~=0));
tau=tau(f:k);
L=L((f-1)*m+1:k*m,:);
k=length(tau);
%-------------------------------------------------------------------------------------%
function [T,C]=ord(t,c,m);

[T,ind]=sort(t);
I=eye(length(T));
I=I(ind,:);
C=kron(I,eye(m))*c;
%-------------------------------------------------------------------------------------%
%-------------------------------------------------------------------------------------%