function [nev,time]=level(fun,z,r,tol,varargin)
%LEVEL plot level curves of surfaces
%   [NEV,TIME] = LEVEL(FUN,Z,R,TOL) plots the curves of level Z of the
%   function FUN in the rectangular region [R(1),R(2)]x[R(3),R(4)] (defined
%   through the vector R=[R(1),R(2),R(3),R(4)]) within a tolerance TOL. It
%   first sets a uniform square grid and then refines the search via
%   adaptive triangulation by considering the sign changes in the values of
%   FUN at the vertices. FUN is a function handle. FUN accepts two real
%   scalar inputs X and Y and returns a real scalar function value F,
%   evaluated at X and Y. NEV is the total number of FUN evaluations
%   required to plot the curves and TIME is the overall CPU time.
%
%   [NEV,TIME] = LEVEL(FUN,Z,R,TOL,VARARGIN) let the user provide values
%   of several internal tolerance parameters different than the default
%   ones. The user is recommend to read [1] prior to modify these values
%   and to follow the prescribed order (default values are bracketed): 
%    St: tolerance for the secant method along cell edges (0.01)
%    Si: maximum number of secant iterations along cell edges (10)
%    Tc: tolerance for adaptive curvature determination (TOL/10)
%    Ts: tolerance for slope test on square cells (0.2)
%    Tt: tolerance for slope test on triangular cells (0.2)
%     p: number of starting square cells along shortest edge (10)
%
%   The algorithm is described in
%   [1] D. Breda, S. Maset and R. Vermiglio, "An adaptive algorithm for
%   efficient computation of level curves of surfaces"
%   Numerical Algorithms xx (2009), pp.xx-xx.
%   Last modified december 24th, 2008.

time=cputime;                    % start computation

Xm=r(1);
XM=r(2);
Ym=r(3);
YM=r(4);

nin=nargin-4;
if nin>0&nin<6
    disp('define all 6 additional parameters!')
    return
end
if nin==0
    St=.01;
    Si=10;
    Tc=tol/10;
    Ts=.2;
    Tt=.2;
    p=10;
else St=varargin{1};
    Si=varargin{2};
    Tc=varargin{3};
    Ts=varargin{4};
    Tt=varargin{5};
    p=varargin{6};
end
    
delta=min(abs(XM-Xm),abs(YM-Ym)); % plane min length
lt=tol/St;                        % triangular cell max edge length
ls=delta/p;                       % square cell edge length

if ls<tol                         % test ls<tol
    ls=tol;                       %
end                               %

if lt>ls/sqrt(2)                  % test lt<=ls/sqrt(2)
    lt=ls/sqrt(2);                %
end                               %

nx=ceil((XM-Xm)/ls);              % number of square cells along x
ny=ceil((YM-Ym)/ls);              % number of square cells along y
XM=Xm+nx*ls;                      % new XM
YM=Ym+ny*ls;                      % new YM
x=Xm:ls:XM;                       % starting square grid x-coords
y=YM:-ls:Ym;                      % starting square grid y-coords

figure(100)                       % set figure
box on
hold on
axis([Xm,XM,Ym,YM])

n=ceil(2*log2(ls/lt));            % max number of subdivision
m=floor((n+1)/2);                 %
d=2^m+1;                          % size of matrix 'S'
lg=ls/(d-1);                      % separation

top=sparse(nx,d);                 % initialize auxiliary matrix 'top'
top(1,1)=fun(Xm,YM)-z;            % on first row of starting square grid
top(nx,d)=fun(XM,YM)-z;           %
for i=1:nx-1                      %
    top(i,d)=fun(x(i+1),YM)-z;    %
    top(i+1,1)=top(i,d);          %           
end                               %

left=sparse(d-1,1);               % initialize auxiliary vector 'left'
left(d-1,1)=fun(Xm,y(2))-z;       %

S=sparse(d,d);                    % initialize matrix 'S' representing
S(1,:)=top(1,:);                  % first square cell
S(2:d,1)=left;                    %
S(d,d)=fun(x(2),y(2))-z;          %

nevs=nx+3;                        % initialize nev on square cells
nevt=0;                           % initialize nev on triangular cells

% analyze each square cell from left to right and top to bottom:
for j=1:ny                                    % y-scan top to bottom
    for i=1:nx                                % x-scan left to right
        
        % test (i,j)-th square cell for possible refinement
        [left,trow,v]=ref(fun,z,tol,lt,Ts,Tt,S,m,d,lg,ls,x(i),y(j),St,Si,Tc);
        
        nevt=nevt+v;                          % update nev on triangular cells
        top(i,:)=trow;                        % update i-th row of matrix 'top'
        
        S=sparse(d,d);                        % update matrix 'S': check if next
        if i<nx                               % square cell is at the beginning
            S(1,:)=top(i+1,:);                % of a new row of the starting grid
            S(:,1)=left;                      %
            S(d,d)=fun(x(i+2),y(j+1))-z;      %
            nevs=nevs+1;                      %
        elseif j<ny                           % or if it is the right-bottom one
            S(1,:)=top(1,:);                  %
            S(d,1)=fun(Xm,y(j+2))-z;          %
            S(d,d)=fun(x(2),y(j+2))-z;        %
            nevs=nevs+2;                      %
        end                                   %
        
    end                                       % end x-scan
end                                           % end y-scan

nev=nevs+nevt;                                % total nev (squares+triangles)
time=cputime-time;                            % end computation
%-------------------------------------------------------------------------------------%
%-------------------------------------------------------------------------------------%

%-------------------------------------------------------------------------------------%
%                                TRIANGULATION FUNCTION                               %
%-------------------------------------------------------------------------------------%
function [right,btm,v]=ref(fun,z,tol,lt,Ts,Tt,S,m,d,lg,ls,xmin,ymax,St,Si,Tc)

x=xmin:lg:xmin+ls;               % x-coords of the square cell
y=ymax:-lg:ymax-ls;              % y-coords of the square cell
v=0;                             % initialize nev inside the square cell

if (sign(S(1,1))==sign(S(1,d)))&(sign(S(1,d))==...   % sign test (see [1])
	sign(S(d,d)))&(sign(S(d,d))==sign(S(d,1)))&...   %
	(min(abs([S(1,1)+S(1,d),S(1,d)+S(d,d),...        % slope test (see [1])
	S(d,d)+S(d,1),S(d,1)+S(1,1)]))>ls*Ts)            %
    % empty slot: possibly plot square if no refinement is required
else c=d/2+.5;                   % square center index
    S(c,c)=fun(x(c),y(c))-z;     % center f evaluations
    v=v+1;                       % update nev
    
    T=zeros(3,2,2*m+2);          % initialize vector 'T' of (3x3)-matrices
    T(:,:,1)=[1,1;1,d;c,c];      % representing the first four triangular cells
    T(:,:,2)=[1,d;d,d;c,c];      %
    T(:,:,3)=[d,d;d,1;c,c];      %
    T(:,:,4)=[d,1;1,1;c,c];      %
    lenT=4;                      % initialize length of 'T'
    
    while lenT>0                 % check if 'T' is empty
        TT=T(:,:,lenT);          % set current triangular cell
        % compute lenght of triangular cell edges
        lT=sqrt([(TT(1,1)-TT(2,1))^2+(TT(1,2)-TT(2,2))^2,...
        (TT(2,1)-TT(3,1))^2+(TT(2,2)-TT(3,2))^2,...
        (TT(3,1)-TT(1,1))^2+(TT(3,2)-TT(1,2))^2])*lg;
        [minlT,im]=min(lT);      % compute shortest edge
        [maxlT,iM]=max(lT);      % compute largest edge
        a=S(TT(1,1),TT(1,2));    % recover vertices values of f from matrix 'S'
        b=S(TT(2,1),TT(2,2));    %
        c=S(TT(3,1),TT(3,2));    %
        if (sign(a)==sign(b))&...              % sign test (see [1])
            (sign(b)==sign(c))&...             %
            (min(abs([(a+b)/lT(1),...          % slope test (see [1])
            (b+c)/lT(2),...                    %
            (c+a)/lT(3)]))>Tt)                 %
            % empty slot: possibly plot triangle if no refinement is required
            T(:,:,lenT)=[];                    % update tail of 'T'
            lenT=lenT-1;                       % update length of 'T'
        elseif ~((sign(a)==sign(b))&...        % sign test (see [1])
            (sign(b)==sign(c)))                %
            if minlT>lt                        % tol test (see [1])
                T1=TT;                         % triangular cell subdivision
                T2=TT;                         %
                jM=mod(iM,3)+1;                %
                xs=(TT(jM,2)+TT(iM,2))/2;      % x-coord of subdivision vertex
                ys=(TT(jM,1)+TT(iM,1))/2;      % y-coord of subdivision vertex

                if S(ys,xs)==0                 % evaluate f at new vertex
                    S(ys,xs)=fun(x(xs),y(ys))-z; % if not previously computed
                    v=v+1;                     % update nev
                end                            %
               
                T1(iM,:)=[ys,xs];              % insert subdivision vertex
                T2(jM,:)=[ys,xs];              % insert subdivision vertex
                T(:,:,lenT)=T1;                % update tail of 'T'
                lenT=lenT+1;                   % update length of 'T'
                T(:,:,lenT)=T2;                % update tail of 'T'
            else p=[x(TT(:,2))',y(TT(:,1))'];  % vertices coords
                ver=[a;b;c];                   % vertices values
                sp=find(sign(ver)==1);         % find values>z
                sm=find(sign(ver)==-1);        % find values<z
                if length(sp)==1               % set edges with sign change
                    ps=p(sp,:);                %
                    pe=p(sm,:);                %
                    fs=ver(sp);                %
                    fe=ver(sm);                %
                else ps=p(sm,:);               %
                    pe=p(sp,:);                %
                    fs=ver(sm);                %
                    fe=ver(sp);                %
                end                            %
                [pz(1,:),v]=secant(fun,z,ps,pe(1,:),fs,fe(1),St,Si,v); % secant 1st edge
                [pz(2,:),v]=secant(fun,z,ps,pe(2,:),fs,fe(2),St,Si,v); % secant 2nd edge
                v=adcur(fun,z,ps,pe,pz,fs,fe,St,Si,Tc,v);              % adapt curvature
                T(:,:,lenT)=[];                % update tail of 'T'
                lenT=lenT-1;                   % update length of 'T'
            end
        elseif minlT>lt                        % slope test false: continue to refine 
            T1=TT;                             % triangular cell subdivision
            T2=TT;                             %
            jM=mod(iM,3)+1;                    %
            xs=(TT(jM,2)+TT(iM,2))/2;          % x-coord of subdivision vertex
            ys=(TT(jM,1)+TT(iM,1))/2;          % y-coord of subdivision vertex

            if S(ys,xs)==0                     % evaluate f at new vertex
                S(ys,xs)=fun(x(xs),y(ys))-z;   % only if not previously computed
                v=v+1;                         % update nev
            end                                %

            T1(iM,:)=[ys,xs];                  % insert subdivision vertex
            T2(jM,:)=[ys,xs];                  % insert subdivision vertex
            T(:,:,lenT)=T1;                    % update tail of 'T'
            lenT=lenT+1;                       % update length of 'T'
            T(:,:,lenT)=T2;                    % update tail of 'T'
        else T(:,:,lenT)=[];                   % update tail of 'T'
            lenT=lenT-1;                       % update length of 'T'
        end
        TT=[];                                 % empty current cell
   end
end
right=S(:,d);                                  % update right vector for 'left'
btm=S(d,:);                                    % update bottom vector for 'top' row
%------------------------------------------------------------------------------------%
%------------------------------------------------------------------------------------%

%------------------------------------------------------------------------------------%
%                            ADAPTIVE CURVATURE FUNCTION                             %
%------------------------------------------------------------------------------------%
function v=adcur(fun,z,ps,pe,pz,fs,fe,St,Si,Tc,v)

wz1=norm(ps-pz(1,:),2)/norm(ps-pe(1,:),2);       % weight 1st edge
wz2=norm(ps-pz(2,:),2)/norm(ps-pe(2,:),2);       % weight 2nd edge
pem=(wz1*pe(1,:)+wz2*pe(2,:))/(wz1+wz2);         % midpoint coords
fem=fun(pem(1),pem(2))-z;                        % evaluate f at midpoint
v=v+1;                                           % update nev
    
if sign(fem)==sign(fe(1))                        % test: 1 edge
    [pzm,v]=secant(fun,z,ps,pem,fs,fem,St,Si,v);   % secant mid edge

    c1=norm([pzm(1)-pz(1,1),pzm(2)-pz(1,2)],2);  % measure height of triangle formed
    c2=norm([pzm(1)-pz(2,1),pzm(2)-pz(2,2)],2);  % by the three zero points
    b=norm([pz(2,1)-pz(1,1),pz(2,2)-pz(1,2)],2); %
    p=(b+c1+c2)/2;                               %
    h=2*sqrt(p*(p-b)*(p-c1)*(p-c2))/b;           %

    if h>Tc                                      % test height for refinement
        v=adcur(fun,z,ps,[pe(1,:);pem],[pz(1,:);pzm],fs,[fe(1);fem],St,Si,Tc,v);
        v=adcur(fun,z,ps,[pe(2,:);pem],[pz(2,:);pzm],fs,[fe(2);fem],St,Si,Tc,v);
    else plot([pz(1,1),pzm(1),pz(2,1)],[pz(1,2),pzm(2),pz(2,2)],'-k') % plot segment
    end
else [pzm1,v]=secant(fun,z,pe(1,:),pem,fe(1),fem,St,Si,v);              % test: 2 edges
    [pzm2,v]=secant(fun,z,pe(2,:),pem,fe(2),fem,St,Si,v);
    v=adcur(fun,z,pe(1,:),[ps;pzm1],[pz(1,:);pzm1],fe(1),[fs;fem],St,Si,Tc,v);
    v=adcur(fun,z,pe(2,:),[ps;pzm2],[pz(2,:);pzm2],fe(2),[fs;fem],St,Si,Tc,v);
end
%------------------------------------------------------------------------------------%
%------------------------------------------------------------------------------------%

%------------------------------------------------------------------------------------%
%                            SECANT METHOD FUNCTION                                  %
%------------------------------------------------------------------------------------%
function [p,v]=secant(fun,z,p1,p2,f1,f2,St,Si,v)

t(1)=0;                                              % parametrization
t(2)=1;                                              %
ffun(1)=f1;                                          %
ffun(2)=f2;                                          %
for i=1:Si                                           % max number of iterations
    
    if (ffun(i+1)*ffun(i)>0)&(i>1)                   % regula falsi
        t(i)=t(i-1);                                 % 
        ffun(i)=ffun(i-1);                           %
    end                                              %
    
    t(i+2)=t(i)-ffun(i)*(t(i+1)-t(i))/(ffun(i+1)-ffun(i)); % new iterate parameter coords
    p=t(i+2)*p2+(1-t(i+2))*p1;                       % new itarate coords
    
    if (abs(t(i+2)-t(i+1))<St)|(abs(t(i+2)-t(i+1))>1)% tolerance on parameter coords
        break                                        %
    end                                              %
    
    if i<Si                                          %
        ffun(i+2)=fun(p(1),p(2))-z;                  % evaluate f at new iterate
        v=v+1;                                       % update nev
    end
end
%------------------------------------------------------------------------------------%
%------------------------------------------------------------------------------------%