function [nev,time]=multilevel(fun,z,r,tol,varargin)
%MULTILEVEL plot level curves of surfaces
%   [NEV,TIME] = MULTILEVEL(FUN,Z,R,TOL) plots the curves of levels given
%   by the entries of the vector Z of the function FUN in the rectangular
%   region [R(1),R(2)]x[R(3),R(4)] (defined through the vector
%   R=[R(1),R(2),R(3),R(4)]) within a tolerance TOL. It first sets a
%   uniform square grid and then refines the search via adaptive
%   triangulation by considering the sign changes in the values of
%   FUN at the vertices. FUN is a function handle. FUN accepts two real
%   scalar inputs X and Y and returns a real scalar function value F,
%   evaluated at X and Y. NEV is the total number of FUN evaluations
%   required to plot the curves and TIME is the overall CPU time.
%
%   [NEV,TIME] = MULTILEVEL(FUN,Z,R,tol,VARARGIN) let the user provide
%   values of several internal tolerance parameters different than the
%   default ones. The user is recommend to read [1] prior to modify these
%   values and to follow the prescribed order (default values are
%   bracketed): 
%    St: tolerance for the secant method along cell edges (0.01)
%    Si: maximum number of secant iterations along cell edges (10)
%    Tc: tolerance for adaptive curvature determination (tol/10)
%    Ts: tolerance for slope test on square cells (0.2)
%    Tt: tolerance for slope test on triangular cells (0.2)
%     p: number of starting square cells along shortest edge (10)
%
%   The algorithm is described in
%   [1] D. Breda, S. Maset and R. Vermiglio, "An adaptive algorithm for
%   efficient computation of level curves of surfaces"
%   Numerical Algorithms xx (2009), pp.xx-xx.
%   Last modified december 24th, 2008.

global lambda A nA model
if model==1
    [lambda,A]=root(-5,-1,1,20);                    % Test 4 in [1]
else [lambda,A]=root([0,1;0.5,0],[0,0;0.5,0],4,20); % Test 5 in [1]
end
nA=size(A,1);
figure(100),plot(real(lambda),imag(lambda),'k*')

nev=0;
time=0;
for i=1:length(z)
    [nevi,timei]=level(fun,z(i),r,tol);
    nev=nev+nevi;
    time=time+timei;
end
%-------------------------------------------------------------------------------------%


%-------------------------------------------------------------------------------------%
%                         CHARACTERISTIC ROOTS FUNCTION                               %
%-------------------------------------------------------------------------------------%
% algorithm described in                                                              %
% D. Breda, S. Maset and R. Vermiglio,                                                %
% "Pseudospectral differencing methods for characteristic roots of                    %
% delay differential equations"                                                       %
% SIAM J. Sci. Comput., Vol. 27 (2), 482-495 (2005)                                   %
%-------------------------------------------------------------------------------------%
function [lambda,AN]=root(L0,L,tau,N)

m=size(L0,1);
k=length(tau);
if k==0
    lambda=eig(L0);
else
    AN=spig(L0,L,m,tau,k,N);
    lambda=eig(AN);
end
%-------------------------------------------------------------------------------------%
function AN=spig(L0,L,m,tau,k,N)

AN=zeros(m*(1+N),m*(1+N));
T=max(tau);
[D,x]=cheb(N,T);
AN(m+1:m*(1+N),1:m*(1+N))=kron(D(2:1+N,1:1+N),eye(m));
for i=1:N+1
    if i==1
        G=L0;
    else
        G=zeros(m);
    end
    for l=1:k
        G=G+L(m*(l-1)+1:m*l,1:m)*elle(i-1,-tau(l),x);
    end
    AN(1:m,m*(i-1)+1:m*i)=G;
end
%-------------------------------------------------------------------------------------%
function [D,x]=cheb(N,T)

if N==0
    D=0;
    x=1;
    return
end
x=cos(pi*(0:N)/N)';
x=T*(x-1)/2;
c=[2;ones(N-1,1);2].*(-1).^(0:N)';
X=repmat(x,1,N+1);
dX=X-X';
D=(c*(1./c)')./(dX+(eye(N+1)));
D=D-diag(sum(D'));
%-------------------------------------------------------------------------------------%
function l=elle(i,y,x)

a=prod(y-x(find(x~=x(i+1))));
b=prod(x(i+1)-x(find(x~=x(i+1))));
l=a/b;
%-------------------------------------------------------------------------------------%
function m=desort(x)

A=[real(x),imag(x)];
A=-sortrows(-A,2);
B=-sortrows(-A,1);
m=B(:,1)+i*B(:,2);
%-------------------------------------------------------------------------------------%
%-------------------------------------------------------------------------------------%